﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Eleicao
{

    class Program
    {
        static void Main(string[] args)
        {

            Eleicao eleicao = Importar(args[0]);
            Relatorios(eleicao);

            Console.ReadKey();
        }

        /// <summary>
        /// Relatórios
        /// </summary>
        /// <param name="eleicao">Eleição que sera usada</param>
        private static void Relatorios(Eleicao eleicao)
        {
            //Número de Vagas
            int numeroVagas = eleicao.Candidatos.Count(c => c.Eleito);
            Console.WriteLine("Número de vagas: " + numeroVagas + "\n");


            //Total de votos
            int totalVotos = eleicao.Candidatos.Sum(c => c.NumeroVotos);
            Console.WriteLine("\nTotal de votos nominais: " + totalVotos);

            //Vereadores Eleitos
            Console.WriteLine("\nVereadores eleitos:");
            eleicao.Candidatos
                .Where(c => c.Eleito == true)
                .Select((c, index) => new { Nome = c.ToString(), index }).ToList()
                .ForEach((c) =>
                    Console.WriteLine(string.Format("{0} - {1}", c.index + 1, c.Nome))
                 );


            //Eleitos se a votação fosse majoritária
            Console.WriteLine("\nCandidatos mais votados (em ordem decrescente de votação e respeitando número de vagas):");
            var eleitosPorNumeroVotos = from canditado in eleicao.Candidatos
                                        orderby canditado.NumeroVotos descending
                                        select canditado.ToString();
            eleitosPorNumeroVotos
                .Take(numeroVagas)
                .Select((Nome, index) => new { Nome, index }).ToList()
                .ForEach((c) =>
                    Console.WriteLine(string.Format("{0} - {1}", c.index + 1, c.Nome))
                 );
        }


        /// <summary>
        /// Importar dados de Eleição
        /// </summary>
        /// <param name="path">Caminho do Arquivo</param>
        /// <returns>Uma eleição</returns>
        private static Eleicao Importar(string path)
        {
            Eleicao eleicao = new Eleicao();
            using (StreamReader sr = new StreamReader(path))
            {
                sr.ReadLine();
                String line;
                while ((line = sr.ReadLine()) != null)
                {
                    String[] values = line.Split(';');

                    //Tratar Candidato
                    Candidato candidato = new Candidato(values[2], values[1]) {
                        Eleito = values[0].Contains("*"),
                        NumeroVotos = Int32.Parse(values[4].Replace(".", "")),
                        Percentual = values[5],
                    };

                    //Tratar Partido e Coligação
                    string[] PartidoEColigacao = values[3].Split('-');
                    string stringPartido = null;
                    string stringColigacao = null;

                    if (PartidoEColigacao.Count() > 1)
                    {
                        stringPartido = PartidoEColigacao[0];
                        stringColigacao = PartidoEColigacao[1];
                    }
                    else
                        stringColigacao = stringPartido = PartidoEColigacao[0];

                    //Tratar Partido
                    Partido partido = eleicao.Partidos.FirstOrDefault(pt => pt.Nome.Equals(stringPartido));
                    bool existePartido = partido != null;
                    partido  = partido  ?? new Partido(stringPartido);
                    partido.Candidatos.Add(candidato);
                    candidato.Partido = partido;
                    if (!existePartido)
                        eleicao.Partidos.Add(partido);

                   
                    Coligacao coligacao = new Coligacao();
                    coligacao.PartidosDaColigacao = stringColigacao;

                    //Coligação já existe
                    if (eleicao.Coligacoes.Any(c => c.PartidosDaColigacao.Equals(stringColigacao)))
                    {
                        Coligacao c = eleicao.Coligacoes.FirstOrDefault(co => co.PartidosDaColigacao.Equals(stringColigacao));
                        c.Candidatos.Add(candidato);
                        candidato.Coligacao = c;
                    }
                    else
                    {
                        //Nova coligação
                        coligacao.Candidatos.Add(candidato);
                        eleicao.Coligacoes.Add(coligacao);
                        candidato.Coligacao = coligacao;
                    }
                    eleicao.Candidatos.Add(candidato);
                }



            }

            return eleicao;
        }
    }
}
